==> skytemple_randomizer-2.0.0-py3-none-any.whl: skytemple_randomizer/randomizer/special_pc.py <==
#  Copyright 2020-2024 Capypara and the SkyTemple Contributors
#
#  This file is part of SkyTemple.
#
#  SkyTemple is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  SkyTemple is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with SkyTemple.  If not, see <https://www.gnu.org/licenses/>.
from random import choice

from skytemple_files.common.types.file_types import FileType
from skytemple_files.common.util import get_binary_from_rom, set_binary_in_rom
from skytemple_files.data.md.protocol import MdProtocol
from skytemple_files.hardcoded.default_starters import HardcodedDefaultStarters
from skytemple_files.list.actor.model import ActorListBin
from skytemple_randomizer.config import MovesetConfig
from skytemple_randomizer.randomizer.abstract import AbstractRandomizer
from skytemple_randomizer.randomizer.util.util import (
    get_allowed_move_ids,
    MoveRoster,
    assert_not_empty,
)
from skytemple_randomizer.status import Status
from skytemple_files.common.i18n_util import _

# Maps actor list indices to special PC indices
ACTOR_TO_SPC_MAPPING = {
    104: [0],
    6: [1],
    5: [3],
    7: [4],
    8: [8],
    9: [7],
    112: [2],
    191: [5],
    136: [6],
    293: [9],
}


class SpecialPcRandomizer(AbstractRandomizer):
    def step_count(self) -> int:
        sp_poke_moves = 1 if self.config["pokemon"]["movesets"] else 0
        if self.config["starters_npcs"]["npcs"]:
            return 1 + sp_poke_moves
        return 0 + sp_poke_moves

    def run(self, status: Status):
        arm9 = bytearray(
            get_binary_from_rom(self.rom, self.static_data.bin_sections.arm9)
        )
        pcs = HardcodedDefaultStarters.get_special_episode_pcs(arm9, self.static_data)

        if self.config["starters_npcs"]["npcs"]:
            status.step(_("Updating special episode Pokémon..."))

            actor_list: ActorListBin = FileType.SIR0.unwrap_obj(
                FileType.SIR0.deserialize(
                    self.rom.getFileByName("BALANCE/actor_list.bin")
                ),
                ActorListBin,
            )

            for i, actor in enumerate(actor_list.list):
                if i in ACTOR_TO_SPC_MAPPING:
                    for bi in ACTOR_TO_SPC_MAPPING[i]:
                        pcs[bi].poke_id = actor.entid

        if self.config["pokemon"]["movesets"] != MovesetConfig.NO:
            status.step(_("Updating special episode Pokémon movesets..."))

            valid_move_ids = get_allowed_move_ids(self.config)
            damaging_move_ids = get_allowed_move_ids(self.config, MoveRoster.DAMAGING)

            md: MdProtocol = FileType.MD.deserialize(
                self.rom.getFileByName("BALANCE/monster.md")
            )
            for pc in pcs:
                pc.move2 = choice(valid_move_ids)
                pc.move3 = choice(valid_move_ids)
                pc.move4 = choice(valid_move_ids)
                pc.do_not_fix_entire_moveset = False
                if self.config["pokemon"]["movesets"] == MovesetConfig.FULLY_RANDOM:
                    pc.move1 = choice(valid_move_ids)
                elif self.config["pokemon"]["movesets"] == MovesetConfig.FIRST_DAMAGE:
                    assert_not_empty(damaging_move_ids)
                    pc.move1 = choice(damaging_move_ids)
                elif self.config["pokemon"]["movesets"] == MovesetConfig.FIRST_STAB:
                    md_entry = md.entries[pc.poke_id]
                    pc.move1 = choice(
                        assert_not_empty(
                            get_allowed_move_ids(
                                self.config, MoveRoster.STAB, md_entry.type_primary
                            )
                        )
                    )

        HardcodedDefaultStarters.set_special_episode_pcs(pcs, arm9, self.static_data)
        set_binary_in_rom(self.rom, self.static_data.bin_sections.arm9, arm9)

        status.done()

==> skytemple_randomizer-2.0.1-py3-none-any.whl: skytemple_randomizer/randomizer/special_pc.py <==
#  Copyright 2020-2024 Capypara and the SkyTemple Contributors
#
#  This file is part of SkyTemple.
#
#  SkyTemple is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  SkyTemple is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with SkyTemple.  If not, see <https://www.gnu.org/licenses/>.
from random import choice

from skytemple_files.common.types.file_types import FileType
from skytemple_files.common.util import get_binary_from_rom, set_binary_in_rom
from skytemple_files.data.md.protocol import MdProtocol
from skytemple_files.hardcoded.default_starters import HardcodedDefaultStarters
from skytemple_files.list.actor.model import ActorListBin
from skytemple_randomizer.config import MovesetConfig
from skytemple_randomizer.randomizer.abstract import AbstractRandomizer
from skytemple_randomizer.randomizer.util.util import (
    get_allowed_move_ids,
    MoveRoster,
    assert_not_empty,
)
from skytemple_randomizer.status import Status
from skytemple_files.common.i18n_util import _

# Maps actor list indices to special PC indices
ACTOR_TO_SPC_MAPPING = {
    104: [0],
    6: [1],
    5: [3],
    7: [4],
    8: [8],
    9: [7],
    112: [2],
    191: [5],
    136: [6],
    293: [9],
}


class SpecialPcRandomizer(AbstractRandomizer):
    def step_count(self) -> int:
        sp_poke_moves = 1 if self.config["pokemon"]["movesets"] else 0
        if self.config["starters_npcs"]["npcs"]:
            return 1 + sp_poke_moves
        return 0 + sp_poke_moves

    def run(self, status: Status):
        arm9 = bytearray(get_binary_from_rom(self.rom, self.static_data.bin_sections.arm9))
        pcs = HardcodedDefaultStarters.get_special_episode_pcs(arm9, self.static_data)

        if self.config["starters_npcs"]["npcs"]:
            status.step(_("Updating special episode Pokémon..."))

            actor_list: ActorListBin = FileType.SIR0.unwrap_obj(
                FileType.SIR0.deserialize(self.rom.getFileByName("BALANCE/actor_list.bin")),
                ActorListBin,
            )

            for i, actor in enumerate(actor_list.list):
                if i in ACTOR_TO_SPC_MAPPING:
                    for bi in ACTOR_TO_SPC_MAPPING[i]:
                        pcs[bi].poke_id = actor.entid

        if self.config["pokemon"]["movesets"] != MovesetConfig.NO:
            status.step(_("Updating special episode Pokémon movesets..."))

            valid_move_ids = get_allowed_move_ids(self.config)
            damaging_move_ids = get_allowed_move_ids(self.config, MoveRoster.DAMAGING)

            md: MdProtocol = FileType.MD.deserialize(self.rom.getFileByName("BALANCE/monster.md"))
            for pc in pcs:
                pc.move2 = choice(valid_move_ids)
                pc.move3 = choice(valid_move_ids)
                pc.move4 = choice(valid_move_ids)
                pc.do_not_fix_entire_moveset = False
                if self.config["pokemon"]["movesets"] == MovesetConfig.FULLY_RANDOM:
                    pc.move1 = choice(valid_move_ids)
                elif self.config["pokemon"]["movesets"] == MovesetConfig.FIRST_DAMAGE:
                    assert_not_empty(damaging_move_ids)
                    pc.move1 = choice(damaging_move_ids)
                elif self.config["pokemon"]["movesets"] == MovesetConfig.FIRST_STAB:
                    md_entry = md.entries[pc.poke_id]
                    pc.move1 = choice(
                        assert_not_empty(get_allowed_move_ids(self.config, MoveRoster.STAB, md_entry.type_primary))
                    )

        HardcodedDefaultStarters.set_special_episode_pcs(pcs, arm9, self.static_data)
        set_binary_in_rom(self.rom, self.static_data.bin_sections.arm9, arm9)

        status.done()
